@@ -107,6 +107,15 @@ cluster.wild.glm<-function(mod, dat, cluster, ci.level = 0.95, impose.null = TRU
                            report = TRUE, prog.bar = TRUE, output.replicates = FALSE,
                            seed=NULL){
   
+  # compensate for bizarre R formula updating bug
+  # thanks to Jason Thorpe for reporting!
+  form.old <- update(mod$formula, 1 ~ 1 )
+  while(form.old != mod$formula){
+    form.old <- mod$formula
+    invisible(mod <- update(mod, formula = .~.))
+  }
+
+  
   if(is.null(seed)==F){                                               # if user supplies a seed, set it
     
     tryCatch(set.seed(seed),
@@ -152,7 +161,7 @@ cluster.wild.glm<-function(mod, dat, cluster, ci.level = 0.95, impose.null = TRU
   }
   
   # check whether there are (automatic) interaction terms
-  interaction <- max(attr(mod$terms,"order"))
+  interaction <- max(attr(mod$terms,"order"), 1)
   
   # do not impose the null for interaction terms
   if( interaction > 1 & impose.null == TRUE){
@@ -320,8 +329,11 @@ cluster.wild.glm<-function(mod, dat, cluster, ci.level = 0.95, impose.null = TRU
     
     comp.fun<-function(vec2, vec1){as.numeric(vec1>vec2)}                            # a simple function comparing v1 to v2
     p.store.s <- t(apply(X = abs(w.store), FUN=comp.fun, MARGIN = 1, vec1 = abs(w))) # compare the BS test stats to orig. result
-    p.store <- 1 - ( colSums(p.store.s) / dim(w.store)[1] )                          # calculate the cluster bootstrap p-value
-    
+    if(dim(p.store.s)[1] == 1){
+      p.store <- 1 - ( sum(p.store.s) / dim(w.store)[1] )                          # calculate the cluster bootstrap p-value
+    }else{
+      p.store <- 1 - ( colSums(p.store.s) / dim(w.store)[1] )                          # calculate the cluster bootstrap p-value
+    }
 
     # compute critical t-statistics for CIs
     crit.t <- apply(X=abs(w.store), MARGIN=2, FUN=quantile, probs=ci.level )
