@@ -69,6 +69,14 @@ cluster.wild.ivreg<-function(mod, dat, cluster, ci.level = 0.95, impose.null = T
                              report = TRUE, prog.bar = TRUE, output.replicates = FALSE,
                              seed=NULL){
   
+  # compensate for bizarre R formula updating bug
+  # thanks to Jason Thorpe for reporting!
+  form.old <- update(mod$formula, 1 ~ 1 )
+  while(form.old != mod$formula){
+    form.old <- mod$formula
+    invisible(mod <- update(mod, formula = .~.))
+  }
+  
   if(is.null(seed)==F){                                               # if user supplies a seed, set it
     
     tryCatch(set.seed(seed),
@@ -297,8 +305,11 @@ cluster.wild.ivreg<-function(mod, dat, cluster, ci.level = 0.95, impose.null = T
     
     comp.fun<-function(vec2, vec1){as.numeric(vec1>vec2)}                              # a simple function comparing v1 to v2
     p.store.s <- t(apply(X = abs(w.store), FUN=comp.fun, MARGIN = 1, vec1 = abs(w)))   # compare the BS test stats to orig. result
-    p.store <- 1 - ( colSums(p.store.s) / dim(w.store)[1] )                            # calculate the cluster bootstrap p-value
-    
+    if(dim(p.store.s)[1] == 1){
+      p.store <- 1 - ( sum(p.store.s) / dim(w.store)[1] )                          # calculate the cluster bootstrap p-value
+    }else{
+      p.store <- 1 - ( colSums(p.store.s) / dim(w.store)[1] )                          # calculate the cluster bootstrap p-value
+    }
 
     # compute critical t-statistics for CIs
     crit.t <- apply(X=abs(w.store), MARGIN=2, FUN=quantile, probs=ci.level )
